@@ -7,7 +7,8 @@ int main(int argc, char **argv)
 	struct isl_ctx *ctx;
 	struct isl_map *map;
 	struct isl_options *options;
-	int exact;
+	isl_printer *p;
+	isl_bool exact;
 
 	options = isl_options_new_with_defaults();
 	assert(options);
@@ -15,19 +16,23 @@ int main(int argc, char **argv)
 
 	ctx = isl_ctx_alloc_with_options(&isl_options_args, options);
 
+	p = isl_printer_to_file(ctx, stdout);
+
 	map = isl_map_read_from_file(ctx, stdin);
 	map = isl_map_transitive_closure(map, &exact);
 	if (!exact)
-		printf("# NOT exact\n");
-	isl_map_print(map, stdout, 0, ISL_FORMAT_ISL);
-	printf("\n");
+		p = isl_printer_print_str(p, "# NOT exact\n");
+	p = isl_printer_print_map(p, map);
+	p = isl_printer_end_line(p);
 	map = isl_map_compute_divs(map);
 	map = isl_map_coalesce(map);
-	printf("# coalesced\n");
-	isl_map_print(map, stdout, 0, ISL_FORMAT_ISL);
-	printf("\n");
+	p = isl_printer_print_str(p, "# coalesced\n");
+	p = isl_printer_print_map(p, map);
+	p = isl_printer_end_line(p);
 	isl_map_free(map);
 
+	isl_printer_free(p);
+
 	isl_ctx_free(ctx);
 
 	return 0;
