@@ -1,6 +1,7 @@
 /*
  * Copyright 2011      INRIA Saclay
  * Copyright 2012-2014 Ecole Normale Superieure
+ * Copyright 2019      Cerebras Systems
  *
  * Use of this software is governed by the MIT license
  *
@@ -8,6 +9,7 @@
  * Parc Club Orsay Universite, ZAC des vignes, 4 rue Jacques Monod,
  * 91893 Orsay, France
  * and Ecole Normale Superieure, 45 rue d’Ulm, 75230 Paris, France
+ * and Cerebras Systems, 175 S San Antonio Rd, Los Altos, CA, USA
  */
 
 #include <isl_ctx_private.h>
@@ -37,7 +39,7 @@ uint32_t isl_local_space_get_hash(__isl_keep isl_local_space *ls)
 		return 0;
 
 	hash = isl_hash_init();
-	space_hash = isl_space_get_hash(ls->dim);
+	space_hash = isl_space_get_full_hash(isl_local_space_peek_space(ls));
 	isl_hash_hash(hash, space_hash);
 	div_hash = isl_mat_get_hash(ls->div);
 	isl_hash_hash(hash, div_hash);
@@ -90,9 +92,10 @@ __isl_give isl_local_space *isl_local_space_alloc(__isl_take isl_space *space,
 	return isl_local_space_alloc_div(space, div);
 }
 
-__isl_give isl_local_space *isl_local_space_from_space(__isl_take isl_space *dim)
+__isl_give isl_local_space *isl_local_space_from_space(
+	__isl_take isl_space *space)
 {
-	return isl_local_space_alloc(dim, 0);
+	return isl_local_space_alloc(space, 0);
 }
 
 __isl_give isl_local_space *isl_local_space_copy(__isl_keep isl_local_space *ls)
@@ -158,24 +161,11 @@ isl_bool isl_local_space_is_set(__isl_keep isl_local_space *ls)
 	return ls ? isl_space_is_set(ls->dim) : isl_bool_error;
 }
 
-/* Do "ls1" and "ls2" have the same space?
- */
-isl_bool isl_local_space_has_equal_space(__isl_keep isl_local_space *ls1,
-	__isl_keep isl_local_space *ls2)
-{
-	if (!ls1 || !ls2)
-		return isl_bool_error;
+#undef TYPE
+#define TYPE	isl_local_space
 
-	return isl_space_is_equal(ls1->dim, ls2->dim);
-}
-
-/* Is the space of "ls" equal to "space"?
- */
-isl_bool isl_local_space_has_space(__isl_keep isl_local_space *ls,
-	__isl_keep isl_space *space)
-{
-	return isl_space_is_equal(isl_local_space_peek_space(ls), space);
-}
+#include "isl_type_has_equal_space_bin_templ.c"
+#include "isl_type_has_space_templ.c"
 
 /* Check that the space of "ls" is equal to "space".
  */
@@ -257,22 +247,42 @@ isl_size isl_local_space_dim(__isl_keep isl_local_space *ls,
 #define TYPE	isl_local_space
 #include "check_type_range_templ.c"
 
-unsigned isl_local_space_offset(__isl_keep isl_local_space *ls,
+/* Return the position of the variables of the given type
+ * within the sequence of variables of "ls".
+ */
+isl_size isl_local_space_var_offset(__isl_keep isl_local_space *ls,
 	enum isl_dim_type type)
 {
 	isl_space *space;
 
+	space = isl_local_space_peek_space(ls);
+	switch (type) {
+	case isl_dim_param:
+	case isl_dim_in:
+	case isl_dim_out:	return isl_space_offset(space, type);
+	case isl_dim_div:	return isl_space_dim(space, isl_dim_all);
+	case isl_dim_cst:
+	default:
+		isl_die(isl_local_space_get_ctx(ls), isl_error_invalid,
+			"invalid dimension type", return isl_size_error);
+	}
+}
+
+/* Return the position of the coefficients of the variables of the given type
+ * within the sequence of coefficients of "ls".
+ */
+unsigned isl_local_space_offset(__isl_keep isl_local_space *ls,
+	enum isl_dim_type type)
+{
 	if (!ls)
 		return 0;
 
-	space = ls->dim;
 	switch (type) {
 	case isl_dim_cst:	return 0;
-	case isl_dim_param:	return 1;
-	case isl_dim_in:	return 1 + space->nparam;
-	case isl_dim_out:	return 1 + space->nparam + space->n_in;
-	case isl_dim_div:
-		return 1 + space->nparam + space->n_in + space->n_out;
+	case isl_dim_param:
+	case isl_dim_in:
+	case isl_dim_out:
+	case isl_dim_div:	return 1 + isl_local_space_var_offset(ls, type);
 	default:		return 0;
 	}
 }
@@ -476,6 +486,66 @@ __isl_keep isl_local *isl_local_space_peek_local(__isl_keep isl_local_space *ls)
 	return ls ? ls->div : NULL;
 }
 
+/* Return a copy of the local variables of "ls".
+ */
+__isl_give isl_local *isl_local_space_get_local(__isl_keep isl_local_space *ls)
+{
+	return isl_local_copy(isl_local_space_peek_local(ls));
+}
+
+/* Return the local variables of "ls".
+ * This may be either a copy or the local variables itself
+ * if there is only one reference to "ls".
+ * This allows the local variables to be modified inplace
+ * if both the local space and its local variables have only a single reference.
+ * The caller is not allowed to modify "ls" between this call and
+ * the subsequent call to isl_local_space_restore_local.
+ * The only exception is that isl_local_space_free can be called instead.
+ */
+static __isl_give isl_local *isl_local_space_take_local(
+	__isl_keep isl_local_space *ls)
+{
+	isl_local *local;
+
+	if (!ls)
+		return NULL;
+	if (ls->ref != 1)
+		return isl_local_space_get_local(ls);
+	local = ls->div;
+	ls->div = NULL;
+	return local;
+}
+
+/* Set the local variables of "ls" to "local",
+ * where the local variables of "ls" may be missing
+ * due to a preceding call to isl_local_space_take_local.
+ * However, in this case, "ls" only has a single reference and
+ * then the call to isl_local_space_cow has no effect.
+ */
+static __isl_give isl_local_space *isl_local_space_restore_local(
+	__isl_take isl_local_space *ls, __isl_take isl_local *local)
+{
+	if (!ls || !local)
+		goto error;
+
+	if (ls->div == local) {
+		isl_local_free(local);
+		return ls;
+	}
+
+	ls = isl_local_space_cow(ls);
+	if (!ls)
+		goto error;
+	isl_local_free(ls->div);
+	ls->div = local;
+
+	return ls;
+error:
+	isl_local_space_free(ls);
+	isl_local_free(local);
+	return NULL;
+}
+
 /* Replace the identifier of the tuple of type "type" by "id".
  */
 __isl_give isl_local_space *isl_local_space_set_tuple_id(
@@ -540,19 +610,19 @@ __isl_give isl_local_space *isl_local_space_set_from_params(
 }
 
 __isl_give isl_local_space *isl_local_space_reset_space(
-	__isl_take isl_local_space *ls, __isl_take isl_space *dim)
+	__isl_take isl_local_space *ls, __isl_take isl_space *space)
 {
 	ls = isl_local_space_cow(ls);
-	if (!ls || !dim)
+	if (!ls || !space)
 		goto error;
 
 	isl_space_free(ls->dim);
-	ls->dim = dim;
+	ls->dim = space;
 
 	return ls;
 error:
 	isl_local_space_free(ls);
-	isl_space_free(dim);
+	isl_space_free(space);
 	return NULL;
 }
 
@@ -563,22 +633,16 @@ error:
 __isl_give isl_local_space *isl_local_space_realign(
 	__isl_take isl_local_space *ls, __isl_take isl_reordering *r)
 {
-	ls = isl_local_space_cow(ls);
-	if (!ls || !r)
-		goto error;
+	isl_local *local;
 
-	ls->div = isl_local_reorder(ls->div, isl_reordering_copy(r));
-	if (!ls->div)
-		goto error;
+	local = isl_local_space_take_local(ls);
+	local = isl_local_reorder(local, isl_reordering_copy(r));
+	ls = isl_local_space_restore_local(ls, local);
 
 	ls = isl_local_space_reset_space(ls, isl_reordering_get_space(r));
 
 	isl_reordering_free(r);
 	return ls;
-error:
-	isl_local_space_free(ls);
-	isl_reordering_free(r);
-	return NULL;
 }
 
 __isl_give isl_local_space *isl_local_space_add_div(
@@ -1315,8 +1379,7 @@ static isl_bool is_linear_div_constraint(__isl_keep isl_local_space *ls,
 	} else {
 		return isl_bool_false;
 	}
-	if (isl_seq_first_non_zero(constraint + pos + 1,
-				    ls->div->n_row - div - 1) != -1)
+	if (isl_seq_any_non_zero(constraint + pos + 1, ls->div->n_row - div - 1))
 		return isl_bool_false;
 	return isl_bool_true;
 }
@@ -1390,9 +1453,10 @@ isl_bool isl_local_space_is_div_equality(__isl_keep isl_local_space *ls,
 	return isl_bool_ok(sign < 0);
 }
 
-/*
- * Set active[i] to 1 if the dimension at position i is involved
- * in the linear expression l.
+/* Return an array of integers, one for each variable of "ls",
+ * with entry i set to 1 if the variable at position i is involved
+ * in the linear expression "l".  This includes variables that appear
+ * in the definition of local variables that appear in "l".
  */
 int *isl_local_space_get_active(__isl_keep isl_local_space *ls, isl_int *l)
 {
@@ -1569,6 +1633,9 @@ __isl_give isl_local_space *isl_local_space_move_dims(
 	enum isl_dim_type dst_type, unsigned dst_pos,
 	enum isl_dim_type src_type, unsigned src_pos, unsigned n)
 {
+	isl_space *space;
+	isl_local *local;
+	isl_size v_src, v_dst;
 	unsigned g_dst_pos;
 	unsigned g_src_pos;
 
@@ -1596,22 +1663,53 @@ __isl_give isl_local_space *isl_local_space_move_dims(
 			"moving dims within the same type not supported",
 			return isl_local_space_free(ls));
 
-	ls = isl_local_space_cow(ls);
-	if (!ls)
-		return NULL;
-
-	g_src_pos = 1 + isl_local_space_offset(ls, src_type) + src_pos;
-	g_dst_pos = 1 + isl_local_space_offset(ls, dst_type) + dst_pos;
+	v_src = isl_local_space_var_offset(ls, src_type);
+	v_dst = isl_local_space_var_offset(ls, dst_type);
+	if (v_src < 0 || v_dst < 0)
+		return isl_local_space_free(ls);
+	g_src_pos = v_src + src_pos;
+	g_dst_pos = v_dst + dst_pos;
 	if (dst_type > src_type)
 		g_dst_pos -= n;
-	ls->div = isl_mat_move_cols(ls->div, g_dst_pos, g_src_pos, n);
-	if (!ls->div)
-		return isl_local_space_free(ls);
-	ls->dim = isl_space_move_dims(ls->dim, dst_type, dst_pos,
+
+	local = isl_local_space_take_local(ls);
+	local = isl_local_move_vars(local, g_dst_pos, g_src_pos, n);
+	ls = isl_local_space_restore_local(ls, local);
+
+	space = isl_local_space_take_space(ls);
+	space = isl_space_move_dims(space, dst_type, dst_pos,
 					src_type, src_pos, n);
-	if (!ls->dim)
+	ls = isl_local_space_restore_space(ls, space);
+
+	return ls;
+}
+
+/* Given a local space (A -> B), return the corresponding local space
+ * (B -> A).
+ */
+__isl_give isl_local_space *isl_local_space_wrapped_reverse(
+	__isl_take isl_local_space *ls)
+{
+	isl_space *space;
+	isl_local *local;
+	isl_size n_in, n_out;
+	unsigned offset;
+
+	space = isl_local_space_peek_space(ls);
+	offset = isl_space_offset(space, isl_dim_set);
+	n_in = isl_space_wrapped_dim(space, isl_dim_set, isl_dim_in);
+	n_out = isl_space_wrapped_dim(space, isl_dim_set, isl_dim_out);
+	if (offset < 0 || n_in < 0 || n_out < 0)
 		return isl_local_space_free(ls);
 
+	space = isl_local_space_take_space(ls);
+	space = isl_space_wrapped_reverse(space);
+	ls = isl_local_space_restore_space(ls, space);
+
+	local = isl_local_space_take_local(ls);
+	local = isl_local_move_vars(local, offset, offset + n_in, n_out);
+	ls = isl_local_space_restore_local(ls, local);
+
 	return ls;
 }
 
@@ -1680,7 +1778,7 @@ __isl_give isl_local_space *isl_local_space_wrap(__isl_take isl_local_space *ls)
 	return ls;
 }
 
-/* Lift the point "pnt", living in the space of "ls"
+/* Lift the point "pnt", living in the (set) space of "ls"
  * to live in a space with extra coordinates corresponding
  * to the local variables of "ls".
  */
@@ -1717,3 +1815,19 @@ error:
 	isl_point_free(pnt);
 	return NULL;
 }
+
+/* Do any of the local variables in "ls" depend on the specified dimensions?
+ */
+isl_bool isl_local_space_involves_dims(__isl_keep isl_local_space *ls,
+	enum isl_dim_type type, unsigned first, unsigned n)
+{
+	isl_local *local;
+	isl_size off;
+
+	off = isl_local_space_var_offset(ls, type);
+	if (off < 0 || isl_local_space_check_range(ls, type, first, n) < 0)
+		return isl_bool_error;
+
+	local = isl_local_space_peek_local(ls);
+	return isl_local_involves_vars(local, off + first, n);
+}
