@@ -50,7 +50,7 @@ uint32_t isl_mat_get_hash(__isl_keep isl_mat *mat)
 	return hash;
 }
 
-struct isl_mat *isl_mat_alloc(struct isl_ctx *ctx,
+__isl_give isl_mat *isl_mat_alloc(isl_ctx *ctx,
 	unsigned n_row, unsigned n_col)
 {
 	int i;
@@ -64,12 +64,14 @@ struct isl_mat *isl_mat_alloc(struct isl_ctx *ctx,
 	mat->block = isl_blk_alloc(ctx, n_row * n_col);
 	if (isl_blk_is_error(mat->block))
 		goto error;
-	mat->row = isl_alloc_array(ctx, isl_int *, n_row);
+	mat->row = isl_calloc_array(ctx, isl_int *, n_row);
 	if (n_row && !mat->row)
 		goto error;
 
-	for (i = 0; i < n_row; ++i)
-		mat->row[i] = mat->block.data + i * n_col;
+	if (n_col != 0) {
+		for (i = 0; i < n_row; ++i)
+			mat->row[i] = mat->block.data + i * n_col;
+	}
 
 	mat->ctx = ctx;
 	isl_ctx_ref(ctx);
@@ -86,7 +88,7 @@ error:
 	return NULL;
 }
 
-struct isl_mat *isl_mat_extend(struct isl_mat *mat,
+__isl_give isl_mat *isl_mat_extend(__isl_take isl_mat *mat,
 	unsigned n_row, unsigned n_col)
 {
 	int i;
@@ -433,12 +435,12 @@ isl_bool isl_mat_is_scaled_identity(__isl_keep isl_mat *mat)
 		return isl_bool_false;
 
 	for (i = 0; i < mat->n_row; ++i) {
-		if (isl_seq_first_non_zero(mat->row[i], i) != -1)
+		if (isl_seq_any_non_zero(mat->row[i], i))
 			return isl_bool_false;
 		if (isl_int_ne(mat->row[0][0], mat->row[i][i]))
 			return isl_bool_false;
-		if (isl_seq_first_non_zero(mat->row[i] + i + 1,
-					    mat->n_col - (i + 1)) != -1)
+		if (isl_seq_any_non_zero(mat->row[i] + i + 1,
+					    mat->n_col - (i + 1)))
 			return isl_bool_false;
 	}
 
@@ -544,10 +546,10 @@ __isl_give isl_mat *isl_mat_aff_direct_sum(__isl_take isl_mat *left,
 	isl_assert(left->ctx, left->n_col >= 1, goto error);
 	isl_assert(left->ctx, right->n_col >= 1, goto error);
 	isl_assert(left->ctx,
-	    isl_seq_first_non_zero(left->row[0]+1, left->n_col-1) == -1,
+	    !isl_seq_any_non_zero(left->row[0]+1, left->n_col-1),
 	    goto error);
 	isl_assert(left->ctx,
-	    isl_seq_first_non_zero(right->row[0]+1, right->n_col-1) == -1,
+	    !isl_seq_any_non_zero(right->row[0]+1, right->n_col-1),
 	    goto error);
 
 	sum = isl_mat_alloc(left->ctx, left->n_row, left->n_col + right->n_col - 1);
@@ -580,8 +582,8 @@ error:
 	return NULL;
 }
 
-static void exchange(struct isl_mat *M, struct isl_mat **U,
-	struct isl_mat **Q, unsigned row, unsigned i, unsigned j)
+static void exchange(__isl_keep isl_mat *M, __isl_keep isl_mat **U,
+	__isl_keep isl_mat **Q, unsigned row, unsigned i, unsigned j)
 {
 	int r;
 	for (r = row; r < M->n_row; ++r)
@@ -594,8 +596,8 @@ static void exchange(struct isl_mat *M, struct isl_mat **U,
 		isl_mat_swap_rows(*Q, i, j);
 }
 
-static void subtract(struct isl_mat *M, struct isl_mat **U,
-	struct isl_mat **Q, unsigned row, unsigned i, unsigned j, isl_int m)
+static void subtract(__isl_keep isl_mat *M, __isl_keep isl_mat **U,
+	__isl_keep isl_mat **Q, unsigned row, unsigned i, unsigned j, isl_int m)
 {
 	int r;
 	for (r = row; r < M->n_row; ++r)
@@ -610,8 +612,8 @@ static void subtract(struct isl_mat *M, struct isl_mat **U,
 	}
 }
 
-static void oppose(struct isl_mat *M, struct isl_mat **U,
-	struct isl_mat **Q, unsigned row, unsigned col)
+static void oppose(__isl_keep isl_mat *M, __isl_keep isl_mat **U,
+	__isl_keep isl_mat **Q, unsigned row, unsigned col)
 {
 	int r;
 	for (r = row; r < M->n_row; ++r)
@@ -646,9 +648,6 @@ __isl_give isl_mat *isl_mat_left_hermite(__isl_take isl_mat *M, int neg,
 		*U = NULL;
 	if (Q)
 		*Q = NULL;
-	if (!M)
-		goto error;
-	M = isl_mat_cow(M);
 	if (!M)
 		goto error;
 	if (U) {
@@ -662,6 +661,13 @@ __isl_give isl_mat *isl_mat_left_hermite(__isl_take isl_mat *M, int neg,
 			goto error;
 	}
 
+	if (M->n_col == 0)
+		return M;
+
+	M = isl_mat_cow(M);
+	if (!M)
+		goto error;
+
 	col = 0;
 	isl_int_init(c);
 	for (row = 0; row < M->n_row; ++row) {
@@ -979,13 +985,13 @@ static isl_stat inv_exchange(__isl_keep isl_mat **left,
 }
 
 static void inv_oppose(
-	struct isl_mat *left, struct isl_mat *right, unsigned row)
+	__isl_keep isl_mat *left, __isl_keep isl_mat *right, unsigned row)
 {
 	isl_seq_neg(left->row[row]+row, left->row[row]+row, left->n_col-row);
 	isl_seq_neg(right->row[row], right->row[row], right->n_col);
 }
 
-static void inv_subtract(struct isl_mat *left, struct isl_mat *right,
+static void inv_subtract(__isl_keep isl_mat *left, __isl_keep isl_mat *right,
 	unsigned row, unsigned i, isl_int m)
 {
 	isl_int_neg(m, m);
@@ -1092,7 +1098,7 @@ error:
 	return NULL;
 }
 
-void isl_mat_col_scale(struct isl_mat *mat, unsigned col, isl_int m)
+void isl_mat_col_scale(__isl_keep isl_mat *mat, unsigned col, isl_int m)
 {
 	int i;
 
@@ -1100,7 +1106,7 @@ void isl_mat_col_scale(struct isl_mat *mat, unsigned col, isl_int m)
 		isl_int_mul(mat->row[i][col], mat->row[i][col], m);
 }
 
-void isl_mat_col_combine(struct isl_mat *mat, unsigned dst,
+void isl_mat_col_combine(__isl_keep isl_mat *mat, unsigned dst,
 	isl_int m1, unsigned src1, isl_int m2, unsigned src2)
 {
 	int i;
@@ -1395,6 +1401,7 @@ __isl_give isl_basic_set *isl_basic_set_preimage(
 	ISL_F_CLR(bset, ISL_BASIC_SET_SORTED);
 	ISL_F_CLR(bset, ISL_BASIC_SET_NORMALIZED_DIVS);
 	ISL_F_CLR(bset, ISL_BASIC_SET_ALL_EQUALITIES);
+	ISL_F_CLR(bset, ISL_BASIC_SET_REDUCED_COEFFICIENTS);
 
 	bset = isl_basic_set_simplify(bset);
 	bset = isl_basic_set_finalize(bset);
@@ -1639,7 +1646,7 @@ __isl_give isl_mat *isl_mat_add_zero_rows(__isl_take isl_mat *mat, unsigned n)
 	return isl_mat_insert_zero_rows(mat, mat->n_row, n);
 }
 
-void isl_mat_col_submul(struct isl_mat *mat,
+void isl_mat_col_submul(__isl_keep isl_mat *mat,
 			int dst_col, isl_int f, int src_col)
 {
 	int i;
@@ -1660,7 +1667,8 @@ void isl_mat_col_add(__isl_keep isl_mat *mat, int dst_col, int src_col)
 			    mat->row[i][dst_col], mat->row[i][src_col]);
 }
 
-void isl_mat_col_mul(struct isl_mat *mat, int dst_col, isl_int f, int src_col)
+void isl_mat_col_mul(__isl_keep isl_mat *mat, int dst_col, isl_int f,
+	int src_col)
 {
 	int i;
 
@@ -1718,7 +1726,7 @@ __isl_give isl_mat *isl_mat_row_neg(__isl_take isl_mat *mat, int row)
 {
 	if (check_row(mat, row) < 0)
 		return isl_mat_free(mat);
-	if (isl_seq_first_non_zero(mat->row[row], mat->n_col) == -1)
+	if (!isl_seq_any_non_zero(mat->row[row], mat->n_col))
 		return mat;
 	mat = isl_mat_cow(mat);
 	if (!mat)
