@@ -14,11 +14,13 @@
 #include <isl/vec.h>
 
 struct isl_scan_callback {
-	int (*add)(struct isl_scan_callback *cb, __isl_take isl_vec *sample);
+	isl_stat (*add)(struct isl_scan_callback *cb,
+		__isl_take isl_vec *sample);
 };
 
-int isl_basic_set_scan(struct isl_basic_set *bset,
+isl_stat isl_basic_set_scan(__isl_take isl_basic_set *bset,
+	struct isl_scan_callback *callback);
+isl_stat isl_set_scan(__isl_take isl_set *set,
 	struct isl_scan_callback *callback);
-int isl_set_scan(__isl_take isl_set *set, struct isl_scan_callback *callback);
 
 #endif
