@@ -93,6 +93,23 @@ isl_stat isl_space_check_is_set(__isl_keep isl_space *space)
 	return isl_stat_ok;
 }
 
+/* Check that "space" is a proper set space,
+ * i.e., the space of a set that is not a parameter domain.
+ */
+isl_stat isl_space_check_is_proper_set(__isl_keep isl_space *space)
+{
+	isl_bool is_params, is_set;
+
+	is_params = isl_space_is_params(space);
+	is_set = isl_space_is_set(space);
+	if (is_params < 0 || is_set < 0)
+		return isl_stat_error;
+	if (is_params || !is_set)
+		isl_die(isl_space_get_ctx(space), isl_error_invalid,
+			"space is not a proper set", return isl_stat_error);
+	return isl_stat_ok;
+}
+
 /* Is the given space that of a map?
  */
 isl_bool isl_space_is_map(__isl_keep isl_space *space)
@@ -121,6 +138,21 @@ static isl_stat isl_space_check_is_map(__isl_keep isl_space *space)
 	return isl_stat_ok;
 }
 
+/* Check that "space" is the space of a set wrapping a map space.
+ */
+isl_stat isl_space_check_is_wrapping(__isl_keep isl_space *space)
+{
+	isl_bool wrapping;
+
+	wrapping = isl_space_is_wrapping(space);
+	if (wrapping < 0)
+		return isl_stat_error;
+	if (!wrapping)
+		isl_die(isl_space_get_ctx(space), isl_error_invalid,
+			"not a product", return isl_stat_error);
+	return isl_stat_ok;
+}
+
 /* Check that "space" is the space of a map
  * where the domain is a wrapped map space.
  */
@@ -362,10 +394,10 @@ isl_size isl_space_wrapped_dim(__isl_keep isl_space *space,
 	return isl_space_dim(isl_space_peek_nested(space, pos), inner);
 }
 
-unsigned isl_space_offset(__isl_keep isl_space *space, enum isl_dim_type type)
+isl_size isl_space_offset(__isl_keep isl_space *space, enum isl_dim_type type)
 {
 	if (!space)
-		return 0;
+		return isl_size_error;
 	return offset(space, type);
 }
 
@@ -383,6 +415,7 @@ static __isl_give isl_space *copy_ids(__isl_take isl_space *dst,
 		id = get_id(src, src_type, i);
 		if (!id)
 			continue;
+		isl_id_free(get_id(dst, dst_type, offset + i));
 		dst = set_id(dst, dst_type, offset + i, isl_id_copy(id));
 		if (!dst)
 			return NULL;
@@ -871,14 +904,14 @@ int isl_space_find_dim_by_id(__isl_keep isl_space *space,
 	enum isl_dim_type type, __isl_keep isl_id *id)
 {
 	int i;
-	int offset;
+	isl_size offset;
 	isl_size n;
 
 	n = isl_space_dim(space, type);
-	if (n < 0 || !id)
+	offset = isl_space_offset(space, type);
+	if (n < 0 || offset < 0 || !id)
 		return -1;
 
-	offset = isl_space_offset(space, type);
 	for (i = 0; i < n && offset + i < space->n_id; ++i)
 		if (space->ids[offset + i] == id)
 			return i;
@@ -890,14 +923,14 @@ int isl_space_find_dim_by_name(__isl_keep isl_space *space,
 	enum isl_dim_type type, const char *name)
 {
 	int i;
-	int offset;
+	isl_size offset;
 	isl_size n;
 
 	n = isl_space_dim(space, type);
-	if (n < 0 || !name)
+	offset = isl_space_offset(space, type);
+	if (n < 0 || offset < 0 || !name)
 		return -1;
 
-	offset = isl_space_offset(space, type);
 	for (i = 0; i < n && offset + i < space->n_id; ++i) {
 		isl_id *id = get_id(space, type, i);
 		if (id && id->name && !strcmp(id->name, name))
@@ -2014,32 +2047,93 @@ error:
 	return NULL;
 }
 
-/* Given a space A -> (B -> C), return the corresponding space
- * A -> (C -> B).
+/* Given a space where the tuple of type "type" is a wrapped map space,
+ * swap domain and range of that wrapped space.
  *
- * If the range tuple is named, then the name is only preserved
- * if B and C are equal tuples, in which case the output
- * of this function is identical to the input.
+ * If the tuple is named, then the name is only preserved
+ * if the nested tuples are equal, in which case the output
+ * of this function is identical to the input, except possibly
+ * for the dimension identifiers.
+ *
+ * Make a reasonable attempt at moving the dimension identifiers
+ * along with the tuples.
  */
-__isl_give isl_space *isl_space_range_reverse(__isl_take isl_space *space)
+__isl_give isl_space *isl_space_reverse_wrapped(__isl_take isl_space *space,
+	enum isl_dim_type type)
 {
+	int pos = type - isl_dim_in;
 	isl_space *nested;
 	isl_bool equal;
+	isl_size n_in;
 
-	if (isl_space_check_range_is_wrapping(space) < 0)
-		return isl_space_free(space);
-
-	nested = isl_space_peek_nested(space, 1);
+	nested = isl_space_peek_nested(space, pos);
 	equal = isl_space_tuple_is_equal(nested, isl_dim_in,
 					nested, isl_dim_out);
 	if (equal < 0)
 		return isl_space_free(space);
 
-	nested = isl_space_take_nested(space, 1);
+	nested = isl_space_take_nested(space, pos);
 	nested = isl_space_reverse(nested);
-	space = isl_space_restore_nested(space, 1, nested);
+	space = isl_space_restore_nested(space, pos, nested);
 	if (!equal)
-		space = isl_space_reset_tuple_id(space, isl_dim_out);
+		space = isl_space_reset_tuple_id(space, type);
+	nested = isl_space_peek_nested(space, pos);
+	n_in = isl_space_dim(nested, isl_dim_in);
+	if (n_in < 0)
+		return isl_space_free(space);
+	space = copy_ids(space, type, 0, nested, isl_dim_in);
+	space = copy_ids(space, type, n_in, nested, isl_dim_out);
+
+	return space;
+}
+
+/* Given a space (A -> B), return the corresponding space
+ * (B -> A).
+ *
+ * If the domain tuple is named, then the name is only preserved
+ * if A and B are equal tuples, in which case the output
+ * of this function is identical to the input, except possibly
+ * for the dimension identifiers.
+ */
+__isl_give isl_space *isl_space_wrapped_reverse(__isl_take isl_space *space)
+{
+	if (isl_space_check_is_wrapping(space) < 0)
+		return isl_space_free(space);
+	space = isl_space_reverse_wrapped(space, isl_dim_set);
+
+	return space;
+}
+
+/* Given a space (A -> B) -> C, return the corresponding space
+ * (B -> A) -> C.
+ *
+ * If the domain tuple is named, then the name is only preserved
+ * if A and B are equal tuples, in which case the output
+ * of this function is identical to the input, except possibly
+ * for the dimension identifiers.
+ */
+__isl_give isl_space *isl_space_domain_reverse(__isl_take isl_space *space)
+{
+	if (isl_space_check_domain_is_wrapping(space) < 0)
+		return isl_space_free(space);
+	space = isl_space_reverse_wrapped(space, isl_dim_in);
+
+	return space;
+}
+
+/* Given a space A -> (B -> C), return the corresponding space
+ * A -> (C -> B).
+ *
+ * If the range tuple is named, then the name is only preserved
+ * if B and C are equal tuples, in which case the output
+ * of this function is identical to the input, except possibly
+ * for the dimension identifiers.
+ */
+__isl_give isl_space *isl_space_range_reverse(__isl_take isl_space *space)
+{
+	if (isl_space_check_range_is_wrapping(space) < 0)
+		return isl_space_free(space);
+	space = isl_space_reverse_wrapped(space, isl_dim_out);
 
 	return space;
 }
@@ -2343,7 +2437,7 @@ __isl_give isl_space *isl_space_bind_set(__isl_take isl_space *space,
 {
 	isl_space *tuple_space;
 
-	if (isl_space_check_is_set(space) < 0)
+	if (isl_space_check_is_proper_set(space) < 0)
 		return isl_space_free(space);
 	tuple_space = isl_multi_id_peek_space(tuple);
 	if (isl_space_check_equal_tuples(tuple_space, space) < 0)
@@ -3225,13 +3319,15 @@ __isl_give isl_space *isl_space_uncurry(__isl_take isl_space *space)
 isl_bool isl_space_has_named_params(__isl_keep isl_space *space)
 {
 	int i;
-	unsigned off;
+	isl_size off;
 
 	if (!space)
 		return isl_bool_error;
 	if (space->nparam == 0)
 		return isl_bool_true;
 	off = isl_space_offset(space, isl_dim_param);
+	if (off < 0)
+		return isl_bool_error;
 	if (off + space->nparam > space->n_id)
 		return isl_bool_false;
 	for (i = 0; i < space->nparam; ++i)
@@ -3269,7 +3365,7 @@ __isl_give isl_space *isl_space_align_params(__isl_take isl_space *space1,
 		goto error;
 
 	exp = isl_parameter_alignment_reordering(space1, space2);
-	exp = isl_reordering_extend_space(exp, space1);
+	isl_space_free(space1);
 	isl_space_free(space2);
 	space1 = isl_reordering_get_space(exp);
 	isl_reordering_free(exp);
