@@ -3,14 +3,14 @@ for (int c0 = -27 * n + 2; c0 <= 1; c0 += 1)
 for (int c0 = 2; c0 <= min(2 * n, n + 29); c0 += 1) {
   if (c0 >= 3) {
     if (2 * n >= c0 + 1) {
-      S4(c0 - c0 / 2 - 1, c0 / 2 + 1);
+      S4((c0 + 1) / 2 - 1, c0 - (c0 + 1) / 2 + 1);
       if (c0 + 2 >= 2 * n) {
         for (int c2 = 1; c2 < -n + c0; c2 += 1)
           S5(-n + c0, n, c2);
       } else if (c0 >= 5) {
-        S4(c0 - c0 / 2 - 2, c0 / 2 + 2);
-        for (int c2 = 1; c2 < c0 - c0 / 2 - 1; c2 += 1)
-          S5(c0 - c0 / 2 - 1, c0 / 2 + 1, c2);
+        S4((c0 + 1) / 2 - 2, c0 - (c0 + 1) / 2 + 2);
+        for (int c2 = 1; c2 < (c0 + 1) / 2 - 1; c2 += 1)
+          S5((c0 + 1) / 2 - 1, c0 - (c0 + 1) / 2 + 1, c2);
       }
     }
     for (int c1 = -c0 + c0 / 2 + 3; c1 <= min(-1, n - c0); c1 += 1) {
@@ -56,14 +56,14 @@ for (int c0 = max(2 * n + 1, -27 * n + 2); c0 <= n + 29; c0 += 1)
   S1(c0 - 1);
 for (int c0 = n + 30; c0 <= 2 * n; c0 += 1) {
   if (2 * n >= c0 + 1) {
-    S4(c0 - c0 / 2 - 1, c0 / 2 + 1);
+    S4((c0 + 1) / 2 - 1, c0 - (c0 + 1) / 2 + 1);
     if (c0 + 2 >= 2 * n) {
       for (int c2 = 1; c2 < -n + c0; c2 += 1)
         S5(-n + c0, n, c2);
     } else {
-      S4(c0 - c0 / 2 - 2, c0 / 2 + 2);
-      for (int c2 = 1; c2 < c0 - c0 / 2 - 1; c2 += 1)
-        S5(c0 - c0 / 2 - 1, c0 / 2 + 1, c2);
+      S4((c0 + 1) / 2 - 2, c0 - (c0 + 1) / 2 + 2);
+      for (int c2 = 1; c2 < (c0 + 1) / 2 - 1; c2 += 1)
+        S5((c0 + 1) / 2 - 1, c0 - (c0 + 1) / 2 + 1, c2);
     }
     for (int c1 = -c0 + c0 / 2 + 3; c1 <= n - c0; c1 += 1) {
       S4(-c1, c0 + c1);
